@@ -192,9 +192,13 @@ def build_cli_parser(do_formatter=False):
                              "duplicate keys are found.")
     '''
     parser.add_argument('--version', action='version', version="\n".join(version_info))
-    parser.add_argument("--force-color", action="store_true", default=False,
+    parser.add_argument("--force-color", dest="tty_mode", const="force", action="store_const",
+                        default=os.environ.get("KSCONF_TTY_COLOR", "auto"),
                         help="Force TTY color mode on.  Useful if piping the output a color-aware "
                              "pager, like 'less -R'")
+    parser.add_argument("--disable-color", dest="tty_mode", const="off", action="store_const",
+                        help="Disable TTY color mode.  This can also be setup as environmental variable: "
+                        "``export KSCONF_TTY_COLOR=off``")
 
     # Logging settings -- not really necessary for simple things like 'diff', 'merge', and 'sort';
     # more useful for 'patch', very important for 'combine'
@@ -203,6 +207,7 @@ def build_cli_parser(do_formatter=False):
 
 
 def cli(argv=None, _unittest=False):
+    # TODO:  Rename '_unitest' to something more appropriate, maybe _exit=True?
     if check_py is not None:
         check_py()
 
@@ -212,7 +217,7 @@ def cli(argv=None, _unittest=False):
 
     args = parser.parse_args(argv)
 
-    ksconf.util.terminal.FORCE_TTY_COLOR = args.force_color
+    ksconf.util.terminal.TTY_COLOR_MODE = args.tty_mode
 
     # This becomes a thing in Python 3.6
     if not hasattr(args, "funct") or args.funct is None:
@@ -226,7 +231,7 @@ def cli(argv=None, _unittest=False):
         if _unittest:
             from ksconf.consts import KSCONF_DEBUG
             os.environ[KSCONF_DEBUG] = "1"
-        sys.stderr.write("Unhandled top-level exception.  {0}\n".format(e))
+        sys.stderr.write("Unhandled top-level exception ({}):  {}\n".format(type(e).__name__, e))
         ksconf.util.debug_traceback()
         return_code = EXIT_CODE_INTERNAL_ERROR
 
@@ -248,7 +253,7 @@ def check_py():
         # Allow   `KSCONF_DEBUG=1 ksconf --version` to run, even if environmental issues exist
         if KSCONF_DEBUG not in os.environ:
             sys.exit(EXIT_CODE_ENV_BUSTED)
-    # Okay, now NEVER call this code again....   (helpful for unit-testing)
+    # Okay, now NEVER call this code again....   (helpful for unit-testing & nested calls)
     globals()["check_py"] = None
 
 
