@@ -21,12 +21,17 @@ import argparse
 import sys
 
 from ksconf.commands import ConfFileType, KsconfCmd, dedent
-from ksconf.conf.parser import PARSECONF_MID_NC, write_conf_stream
+from ksconf.conf.parser import PARSECONF_MID_NC, conf_attr_boolean, write_conf_stream
 from ksconf.consts import EXIT_CODE_SUCCESS
-from ksconf.filter import FilteredList, FilterListWildcard, create_filtered_list
+from ksconf.filter import FilteredList, FilteredListWildcard, create_filtered_list
 from ksconf.util.completers import conf_files_completer
 
 
+def is_disabled(attributes):
+    # type: (dict) -> bool
+    return conf_attr_boolean(attributes.get("disabled", False))
+
+
 class FilterCmd(KsconfCmd):
     help = "A stanza-aware GREP tool for conf files"
     description = dedent("""
@@ -121,6 +126,18 @@ class FilterCmd(KsconfCmd):
             PATTERN supports the special ``file://`` syntax.""")
         '''
 
+        pg_eod = pg_sel.add_mutually_exclusive_group()
+        pg_eod.add_argument("-e", "--enabled-only", action="store_true",
+                            help=dedent("""
+            Keep only enabled stanzas.  Any stanza containing ``disabled = 1`` will be removed.
+            The value of ``disabled`` is assumed to be false by default.
+            """))
+
+        pg_eod.add_argument("-d", "--disabled-only", action="store_true",
+                            help=dedent("""
+            Keep disabled stanzas only.
+            The value of the `disabled` attribute is interpreted as a boolean. """))
+
         pg_con = parser.add_argument_group("Attribute selection", dedent("""\
             Include or exclude attributes passed through.
             By default, all attributes are preserved.
@@ -149,11 +166,18 @@ class FilterCmd(KsconfCmd):
         self.attr_presence_filters = create_filtered_list(args.match, flags)
         self.attr_presence_filters.feedall(args.attr_present)
 
+        if args.enabled_only:
+            self.disabled_filter = lambda attrs: not is_disabled(attrs)
+        elif args.disabled_only:
+            self.disabled_filter = is_disabled
+        else:
+            self.disabled_filter = lambda attrs: True
+
         if args.keep_attrs or args.reject_attrs:
-            self.attrs_keep_filter = FilterListWildcard(flags)
+            self.attrs_keep_filter = FilteredListWildcard(flags)
             for attrs in args.keep_attrs:
                 self.attrs_keep_filter.feedall(attrs.split(" "))
-            self.attrs_reject_filter = FilterListWildcard(FilteredList.INVERT | flags)
+            self.attrs_reject_filter = FilteredListWildcard(FilteredList.INVERT | flags)
             for attrs in args.reject_attrs:
                 self.attrs_reject_filter.feedall(attrs.split(" "))
         else:
@@ -161,7 +185,11 @@ class FilterCmd(KsconfCmd):
             self.filter_attrs = lambda x: x
 
     def _test_stanza(self, stanza, attributes):
-        if self.stanza_filters.match(stanza):
+        if self.stanza_filters.match_stanza(stanza):
+            # Exclude based on value of 'disabled' attribute
+            if not self.disabled_filter(attributes):
+                return False
+
             # If there are no attribute level filters, automatically keep (preserves empty stanzas)
             if not self.attr_presence_filters.has_rules:
                 return True
