@@ -125,6 +125,10 @@ class PromoteCmd(KsconfCmd):
                           action="store_const",
                           dest="mode", const="summary",
                           help="Summarize content that could be promoted.")
+        grp1.add_argument("--diff", "-d",
+                          action="store_const",
+                          dest="mode", const="diff",
+                          help="Show the diff of what would be promoted.")
 
         parser.add_argument("--verbose", action="store_true", default=False,
                             help="Enable additional output.")
@@ -216,7 +220,7 @@ class PromoteCmd(KsconfCmd):
             # Allow local.meta -> default.meta without --force or a warning message
             pass
         elif bn_source != bn_target:
-            if args.mode == "summary":
+            if args.mode in ("summary", "diff"):
                 pass
             # Todo: Allow for interactive prompting when in interactive but not force mode.
             elif args.force:
@@ -242,13 +246,19 @@ class PromoteCmd(KsconfCmd):
         # Prep filters and determine if there's any automatic work that can be done
         if self.prep_filters(args):
             # Run filter and then return control back
-            cfg_src, cfg_tgt = self._do_promote_list(cfg_src, cfg_tgt, args)
+            delta, cfg_src, cfg_tgt = self._do_promote_list(cfg_src, cfg_tgt, args)
+        else:
+            delta = compare_cfgs(cfg_tgt, cfg_src, replace_level="key")
+            delta = [op for op in delta if op.tag != DIFF_OP_DELETE]
+
+        if args.mode == "diff":
+            self.stderr.write("\n")
+            show_diff(self.stderr, delta)
+            return
 
         if args.mode in ("ask", "summary"):
             # Show a summary of how many new stanzas would be copied across; how many key changes.
             # And either accept all (batch) or pick selectively (batch)
-            delta = compare_cfgs(cfg_tgt, cfg_src, allow_level0=False)
-            delta = [op for op in delta if op.tag != DIFF_OP_DELETE]
             summarize_cfg_diffs(delta, self.stderr)
             if args.mode == "summary":
                 return
@@ -312,6 +322,11 @@ class PromoteCmd(KsconfCmd):
         if args.stanza:
             return True
 
+    def apply_filters(self, delta, invert_match=False):
+        for op in delta:
+            if self.stanza_filters.match_stanza(op.location.stanza) ^ invert_match:
+                yield op
+
     @staticmethod
     def combine_stanza(a, b):
         if a is None:
@@ -390,7 +405,7 @@ class PromoteCmd(KsconfCmd):
         # Todo:  IMPLEMENT A MANUAL MERGE/DIFF HERE:
         # What ever is migrated, move it OUT of cfg_src, and into cfg_tgt
 
-        diff = compare_cfgs(cfg_tgt, cfg_src, allow_level0=False)
+        diff = compare_cfgs(cfg_tgt, cfg_src, replace_level="key")
         for op in diff:
             if op.tag == DIFF_OP_DELETE:
                 # This is normal.   Only changed attributes are copied & updated in local.
@@ -440,21 +455,21 @@ class PromoteCmd(KsconfCmd):
     def _do_promote_list(self, cfg_src, cfg_tgt, args):
         out_src = deepcopy(cfg_src)
         out_cfg = deepcopy(cfg_tgt)
-        diff = [op for op in compare_cfgs(cfg_tgt, cfg_src, allow_level0=False)
+        diff = compare_cfgs(cfg_tgt, cfg_src, replace_level="key")
+        diff = [op for op in self.apply_filters(diff, args.invert_match)
                 if op.tag in (DIFF_OP_INSERT, DIFF_OP_REPLACE)]
         for op in diff:
-            if self.stanza_filters.match(op.location.stanza) ^ args.invert_match:
-                if args.verbose:
-                    show_diff(self.stdout, [op])
-                if isinstance(op.location, DiffStanza):
-                    # Move entire stanza
-                    out_cfg[op.location.stanza] = self.combine_stanza(op.a, op.b)
+            if args.verbose:
+                show_diff(self.stdout, [op])
+            if isinstance(op.location, DiffStanza):
+                # Move entire stanza
+                out_cfg[op.location.stanza] = self.combine_stanza(op.a, op.b)
+                del out_src[op.location.stanza]
+            else:
+                # Move key
+                out_cfg[op.location.stanza][op.location.key] = op.b
+                del out_src[op.location.stanza][op.location.key]
+                # If last remaining key in the src stanza?  Then delete the entire stanza
+                if not out_src[op.location.stanza]:
                     del out_src[op.location.stanza]
-                else:
-                    # Move key
-                    out_cfg[op.location.stanza][op.location.key] = op.b
-                    del out_src[op.location.stanza][op.location.key]
-                    # If last remaining key in the src stanza?  Then delete the entire stanza
-                    if not out_src[op.location.stanza]:
-                        del out_src[op.location.stanza]
-        return (out_src, out_cfg)
+        return (diff, out_src, out_cfg)
