@@ -49,30 +49,28 @@ class DiffHeader(object):
         return "{0:50} {1}".format(self.name, ts)
 
 
-def compare_stanzas(a, b, stanza_name, preserve_empty=False):
-    if preserve_empty:
-        def is_empty(v):
-            return v is None
-    else:
-        def is_empty(v):
-            return not v
-
+def compare_stanzas(a, b, stanza_name, replace_level="global"):
+    """
+    :param replace_level: If a and b have no common keys, is a single stanza-level
+                      'replace' is issue unless ``replace_level="key"``
+    :type replace_level: bool
+    """
     if a == b:
         return [DiffOp(DIFF_OP_EQUAL, DiffStanza("stanza", stanza_name), a, b)]
-    elif is_empty(b):
+    elif b is None:
         # A only
         return [DiffOp(DIFF_OP_DELETE, DiffStanza("stanza", stanza_name), a, None)]
-    elif is_empty(a):
+    elif a is None:
         # B only
         return [DiffOp(DIFF_OP_INSERT, DiffStanza("stanza", stanza_name), None, b)]
     else:
-        return list(_compare_stanzas(a, b, stanza_name))
+        return list(_compare_stanzas(a, b, stanza_name, replace_level))
 
 
-def _compare_stanzas(a, b, stanza_name):
+def _compare_stanzas(a, b, stanza_name, replace_level):
     kv_a, kv_common, kv_b = _cmp_sets(list(a.keys()), list(b.keys()))
 
-    if not kv_common:
+    if replace_level in ("global", "stanza") and not kv_common:
         # No keys in common, just swap
         yield DiffOp(DIFF_OP_REPLACE, DiffStanza("stanza", stanza_name), a, b)
         return
@@ -91,14 +89,35 @@ def _compare_stanzas(a, b, stanza_name):
             yield DiffOp(DIFF_OP_REPLACE, DiffStzKey("key", stanza_name, key), a_, b_)
 
 
-def compare_cfgs(a, b, allow_level0=True, preserve_empty=False):
+def compare_cfgs(a, b, replace_level="global"):
     '''
-    Return list of 5-tuples describing how to turn a into b.
+    Calculate a set of deltas which describes how to transform a into b.
 
-    .. note:: The `Opcode` tags borrowed from :class:`SequenceMatcher` class in the :mod:`difflib`
-              standard Python module.
+    :param a: the first/original configuration entity
+    :type a: dict
+    :param b: the second/target configuration entity
+    :type b: dict
+    :param replace_level: The highest level 'replace' event that can be returned.
+        Acceptable values are ``global``, ``stanza``, and ``key``.
+        These examples may help:
 
-    Each tuple takes the form:
+            *   Using 'global' with identical inputs will report a single global-level equal op.
+            *   Using 'stanza' with identical inputs will return all stanzas as equal.
+            *   Using 'key' will ensure that two stanzas with no common keys will be reported in
+                terms of key changes.  Whereas 'global' or 'stanza' would result in a single giant replace op.
+
+    :type replace_level: str: ``global``, ``stanza``, or ``key``
+    :return: a sequence of differences in tuples
+    :rtype: [DiffOp]
+
+    .. note:: The :py:class:`DiffOp` output idea was borrowed from
+              :class:`SequenceMatcher` class in the :mod:`difflib`
+              in the standard Python module.
+
+    This function returns a sequence of 5 element tuples describing the
+    transformation based on the detail level specified in `replace_level`.
+
+    Each :py:class:`DiffOp` (named tuple) takes the form:
 
         (tag, location, a, b)
 
@@ -113,27 +132,38 @@ def compare_cfgs(a, b, allow_level0=True, preserve_empty=False):
     'equal'	    same values in both
     =========   ============================================
 
-    *location* is a tuple that can take the following forms:
+    *location* is a namedtuple that can take the following forms:
 
-    ===================  ===============================================================
-    Tuple form           Description
-    ===================  ===============================================================
-    `(0)`                Global file level context (e.g., both files are the same)
-    `(1, stanza)`        Stanzas are the same, or completely different (no shared keys)
-    `(2, stanza, key)`   Key level, indicating
-    ===================  ===============================================================
+    ====================== ========== ==============================================================
+    Tuple form             Type       Description
+    ====================== ========== ==============================================================
+    `("global")`           DiffGlobal Global file level context (e.g., both files are the same)
+    `("stanza", stanza)`   DiffStanza Stanzas are the same, or completely different (no shared keys)
+    `("key", stanza, key)` DiffStzKey Key level change
+    ====================== ========== ==============================================================
 
+    .. versionchanged:: v0.8.8
+        The ``preserve_empty`` argument was origionally introduced to preserve backwards
+        compatibility, but it ended up introducing new bugs.
+        Additionally, no use cases were found where better to automatically discarding empty stanzas.
 
-    Possible alternatives:
-
-    https://dictdiffer.readthedocs.io/en/latest/#dictdiffer.patch
+    .. versionchanged:: v0.8.8
+        The ``allow_level0`` argument was replaced with ``replace_level``.
+        Instead of using ``allow_level0=False`` use ``replace_level="stanza"``.
+        At the same time an new feature was added to support ``replace_level="key"``.
+        The default behavior remains the same.
 
     '''
+    # Possible alternatives:
+    # https://dictdiffer.readthedocs.io/en/latest/#dictdiffer.patch
+
+    if replace_level not in ("global", "stanza", "key"):
+        raise TypeError("Invalid value given for 'replace_level'. Choose 'global', 'stanza', or 'key'")
 
     delta = []
 
     # Level 0 - Compare entire file
-    if allow_level0:
+    if replace_level == "global":
         stanza_a, stanza_common, stanza_b = _cmp_sets(list(a.keys()), list(b.keys()))
         if a == b:
             return [DiffOp(DIFF_OP_EQUAL, DiffGlobal("global"), a, b)]
@@ -154,7 +184,7 @@ def compare_cfgs(a, b, allow_level0=True, preserve_empty=False):
         all_stanzas = list(all_stanzas)
     all_stanzas = sorted(all_stanzas)
     for stanza in all_stanzas:
-        delta.extend(compare_stanzas(a.get(stanza), b.get(stanza), stanza, preserve_empty))
+        delta.extend(compare_stanzas(a.get(stanza), b.get(stanza), stanza, replace_level))
     return delta
 
 
@@ -251,7 +281,6 @@ def show_diff(stream, diffs, headers=None):
                     stream.write("{0}[{1}]\n".format(prefix_, stanza_))
                 for x, y in sorted(six.iteritems(value)):
                     write_key(x, y, prefix_)
-                stream.write("\n")
             else:
                 write_key(key, value, prefix_)
 
@@ -297,22 +326,23 @@ def show_diff(stream, diffs, headers=None):
 
     last_stanza = None
     for op in diffs:
-        if isinstance(op.location, DiffStanza):
-            if op.tag in (DIFF_OP_DELETE, DIFF_OP_REPLACE):
-                show_value(op.a, op.location.stanza, None, "-")
-            if op.tag in (DIFF_OP_INSERT, DIFF_OP_REPLACE):
-                show_value(op.b, op.location.stanza, None, "+")
-            continue  # pragma: no cover  (peephole optimization)
 
         if op.location.stanza != last_stanza:
             if last_stanza is not None:
                 # Line break after last stanza
                 stream.write("\n")
                 stream.flush()
-            if op.location.stanza is not GLOBAL_STANZA:
+            if op.location.stanza is not GLOBAL_STANZA and not isinstance(op.location, DiffStanza):
                 stream.write(" [{0}]\n".format(op.location.stanza))
             last_stanza = op.location.stanza
 
+        if isinstance(op.location, DiffStanza):
+            if op.tag in (DIFF_OP_DELETE, DIFF_OP_REPLACE):
+                show_value(op.a, op.location.stanza, None, "-")
+            if op.tag in (DIFF_OP_INSERT, DIFF_OP_REPLACE):
+                show_value(op.b, op.location.stanza, None, "+")
+            continue
+
         if op.tag == DIFF_OP_INSERT:
             show_value(op.b, op.location.stanza, op.location.key, "+")
         elif op.tag == DIFF_OP_DELETE:
@@ -354,3 +384,39 @@ def reduce_stanza(stanza, keep_attrs):
     :return: a reduced copy of ``stanza``.
     """
     return {attr: value for attr, value in six.iteritems(stanza) if attr in keep_attrs}
+
+
+def write_diff_as_json(delta, stream, **dump_args):
+    # XXX: Eventually support reversing this to import a json "patch"; and add an apply/reverse command.
+    import json
+    import sys
+
+    from ksconf import __vcs_info__, __version__
+    record = {
+        "schema_version": 1,
+        "software": {
+            "name": "ksconf",
+            "version": [__version__, __vcs_info__],
+            "command": sys.argv,
+            "cwd": os.getcwd(),
+        }
+    }
+    record["records"] = [diff_obj_json_format(op) for op in delta]
+    json.dump(record, stream, **dump_args)
+
+
+def diff_obj_json_format(o):
+    if isinstance(o, DiffOp):
+        o = {
+            "tag": o.tag,
+            "location": diff_obj_json_format(o.location),
+            "a": o.a,
+            "b": o.b,
+        }
+    elif isinstance(o, DiffGlobal):
+        o = {"type": o.type}
+    elif isinstance(o, DiffStanza):
+        o = {"type": o.type, "stanza": o.stanza}
+    elif isinstance(o, DiffStzKey):
+        o = {"type": o.type, "stanza": o.stanza, "key": o.key}
+    return o
