@@ -7,6 +7,12 @@ from fnmatch import fnmatch
 
 from ksconf.util.file import relwalk
 
+try:
+    # PY3 with no-op fallback for older versions
+    from os import PathLike
+except ImportError:
+    PathLike = object
+
 """
 
 LayerRootBase has one or more 'Layer', each layer has one or more 'File's.
@@ -116,21 +122,24 @@ class LayerConfig(object):
     def __init__(self):
         # Set defaults
         self.follow_symlink = False
-        self.blacklist_files = re.compile(r"\.(bak|swp)$")
-        self.blacklist_dirs = {".git"}
+        self.block_files = re.compile(r"\.(bak|swp)$")
+        self.block_dirs = {".git"}
 
 
 class LayerRootBase(object):
     """ All 'path's here are relative to the ROOT. """
 
-    class File(object):
-        __slots__ = ["layer", "relative_path", "size", "mtime", "_cache"]
+    class File(PathLike):
+        __slots__ = ["layer", "relative_path", "_stat"]
 
-        def __init__(self, layer, relative_path, size=None, mtime=None):
+        def __init__(self, layer, relative_path, stat=None):
             self.layer = layer
             self.relative_path = relative_path
-            self.size = size
-            self.mtime = mtime
+            self._stat = stat
+
+        def __fspath__(self):
+            # type: () -> str
+            return self.physical_path
 
         @property
         def physical_path(self):
@@ -140,6 +149,20 @@ class LayerRootBase(object):
         def logical_path(self):
             return _path_join(self.layer.logical_path, self.relative_path)
 
+        @property
+        def stat(self):
+            if self._stat is None:
+                self._stat = os.stat(self.physical_path)
+            return self._stat
+
+        @property
+        def size(self):
+            return self.stat.st_size
+
+        @property
+        def mtime(self):
+            return self.stat.st_mtime
+
     class Layer(object):
         """ Basic layer Container:   Connects logical and physical paths. """
         __slots__ = ["name", "root", "logical_path", "physical_path", "config", "_file_cls"]
@@ -157,9 +180,9 @@ class LayerRootBase(object):
             # In the simple case, this is good enough.   Some subclasses will need to override
             for (root, dirs, files) in relwalk(_path_join(self.root, self.physical_path),
                                                followlinks=self.config.follow_symlink):
-                files = [f for f in files if not self.config.blacklist_files.search(f)]
+                files = [f for f in files if not self.config.block_files.search(f)]
                 for d in list(dirs):
-                    if d in self.config.blacklist_dirs:
+                    if d in self.config.block_dirs:
                         dirs.remove(d)
                 yield (root, dirs, files)
 
@@ -344,16 +367,22 @@ class DotDLayerRoot(LayerRootBase):
             return l is self._root_layer or layer_filter(l)
         return super(DotDLayerRoot, self).apply_filter(fltr)
 
-    def set_root(self, root):
+    def set_root(self, root, follow_symlinks=None):
         """ Set a root path, and auto discover all '.d' directories.
 
-        Note:  We currently only support '.d/<layer>' directories, so something like
+        Note:  We currently only support '.d/<layer>' directories, a file like
         `default.d/10-props.conf` won't be handled here.
         """
         Layer, File = self.Layer, self.File
-        for (top, dirs, files) in relwalk(root, topdown=False):
+        if follow_symlinks is None:
+            follow_symlinks = self.config.follow_symlink
+
+        for (top, dirs, files) in relwalk(root, topdown=False, followlinks=follow_symlinks):
             del files
-            mount_mo = self.mount_regex.match(top)
+
+            top_dirname, top_basename = os.path.split(top)
+            mount_mo = self.mount_regex.match(top_basename)
+
             if mount_mo:
                 for dir_ in dirs:
                     dir_mo = self.layer_regex.match(dir_)
@@ -361,16 +390,25 @@ class DotDLayerRoot(LayerRootBase):
                         # XXX: Nested layers breakage, must substitute multiple ".d" folders in `top`
                         layer = Layer(dir_mo.group("layer"),
                                       root,
-                                      physical=os.path.join(os.path.basename(top), dir_),
-                                      logical=mount_mo.group("realname"),
+                                      physical=os.path.join(top, dir_),
+                                      logical=os.path.join(top_dirname, mount_mo.group("realname")),
                                       config=self.config,
                                       file_cls=File)
                         self.add_layer(layer)
                         self._mount_points[top].append(dir_)
                     else:
-                        # XXX: Give the use the option of logging the near-matches (could indicate a
+                        # XXX: Give the user the option of logging the near-matches (could indicate a
                         # problem in the config, or could be some other legit directory structure)
+                        '''
+                        print("LAYER NEAR MISS:  {} looks like a mount point, but {} doesn't "
+                              "follow the expected convention".format(top, dir_))
+                        '''
                         pass
+            elif top.endswith(".d"):
+                '''
+                print("MOUNT NEAR MISS:  {}".format(top))
+                '''
+                pass
 
         # XXX: Adding <root> should be skipped if (and only if) root itself if a '.d' folder
         # Very last operation, add the top directory as the final layer (lowest rank)
