@@ -30,6 +30,20 @@ def get_merged_conf(app_dir, conf, *layers):
     return merge_conf_dicts(*confs)
 
 
+def normalize_directory_mtime(path):
+    """ Walk a tree and update the directory modification times to match the
+    newest time of the children.  This results in a more predictable behavior
+    over multiple executions.
+    """
+    for (root, dirs, files) in os.walk(path, topdown=False):
+        nodes = dirs + files
+        if not nodes:
+            # Empty directories are somewhat unlikely.  We'll see
+            continue
+        mtime = max(os.stat(os.path.join(root, n)).st_mtime for n in nodes)
+        os.utime(root, (mtime, mtime))
+
+
 class PackagingException(Exception):
     pass
 
@@ -59,7 +73,7 @@ class AppPackager(object):
         return self._var_magic.expand(value)
 
     def expand_new_only(self, value):
-        """ Expand a variable but return None if no substitution occurred
+        """ Expand a variable but return False if no substitution occurred
 
         :param str value:  String that main contain ``{{variable}}`` substitution.
         :return:  Expanded value if variables were expanded, else False
@@ -87,7 +101,7 @@ class AppPackager(object):
             raise PackagingException("Issue calling 'combine' internally during app build....")
 
     def blocklist(self, patterns):
-        # XXX: Rewrite explicitly blocklist '.git' dir, because '*.git' wasn't working here. :=(
+        # XXX: Rewrite explicitly blocklist '.git' dir, because '.git*' wasn't working here. :=(
 
         # For now we just delete files out the build directory.  Not sophisticated, but it works
         # Do we need relwalker here?  relwalk
@@ -110,6 +124,7 @@ class AppPackager(object):
                         break
 
     def merge_local(self):
+        # XXX:  Rename this "promote_local()" ?
         """
         Find everything in local, if it has a corresponding file in default, merge.
         """
@@ -210,6 +225,7 @@ class AppPackager(object):
         else:
             self.output.write("Creating archive:  {}\n".format(filename))
 
+        normalize_directory_mtime(self.app_dir)
         spl = tarfile.open(filename, mode="w:gz")
         try:
             spl.add(self.app_dir, arcname=self.app_name)
